=begin

    Copyright The Resistance (C) 1995-2011

    Author: blass
    blass[at]sdf.lonestar.org

    Badass 0.1 alpha

    This file is part of Badass.

    Badass is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Badass is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Badass.  If not, see <http://www.gnu.org/licenses/>.

=end

module Racket
  module L4
    # Internet Group Management Protocol, Version 1
    #
    # RFC1112 (http://www.faqs.org/rfcs/rfc1112.html)
    # 
    class IGMPv1 < RacketPart
      # Version (defaults to 1)
      unsigned :version, 4, { :default => 1 }
      # Type
      unsigned :type, 4
      # Unused
      unsigned :unused, 8
      # Checksum
      unsigned :checksum, 16
      # Group Address
      octets :gaddr, 32
      # Payload
      rest :payload
      
      # Check the checksum for this IGMP message
      def checksum?
        self.checksum == 0 || (self.checksum == compute_checksum)
      end
      
      # Compute and set the checkum for this IGMP message
      def checksum!
        self.checksum = compute_checksum
      end
      
      # Do whatever 'fixing' is neccessary in preparation
      # for being sent
      def fix!
        self.checksum!
      end
      
      private
      def compute_checksum
        # The checksum is the 16-bit one's complement of the one complement sum
        # of the 8-octet IGMP message.  For computing the checksum, the checksum
        # field is zeroed.
        tmp = []
        tmp << ((((self.version << 4) | self.type) << 8) | self.unused)
        tmp << 0
        tmp << L3::Misc.ipv42long(self.gaddr)
        tmp << self.payload
        L3::Misc.checksum(tmp.pack("nnNa*"))
      end
    end
  end
end
# vim: set ts=2 et sw=2:
